"""
This module contains classes for accessing `HERE Matrix Routing API <https://developer.here.com/documentation/matrix-routing-api/8.3.0/dev_guide/index.html>`_.
"""  # noqa E501
from datetime import datetime
from typing import Any, Dict, List, Optional, Union

import requests

from here_location_services.platform.auth import Auth

from .apis import Api
from .config.base_config import Truck
from .config.matrix_routing_config import (
    AutoCircleRegion,
    AvoidBoundingBox,
    BoundingBoxRegion,
    CircleRegion,
    PolygonRegion,
    WorldRegion,
)
from .exceptions import ApiError


class MatrixRoutingApi(Api):
    """A class to access Matrix Routing API."""

    def __init__(
        self,
        api_key: Optional[str] = None,
        auth: Optional[Auth] = None,
        proxies: Optional[dict] = None,
        country: str = "row",
    ):
        super().__init__(api_key, auth=auth, proxies=proxies, country=country)
        self._base_url = f"https://matrix.router.{self._get_url_string()}"

    def __send_post_request(
        self,
        async_req: str,
        origins: List[Dict],
        region_definition: Union[
            CircleRegion,
            BoundingBoxRegion,
            PolygonRegion,
            AutoCircleRegion,
            WorldRegion,
        ],
        destinations: Optional[List[Dict]] = None,
        profile: Optional[str] = None,
        departure_time: Optional[Union[datetime, str]] = None,
        routing_mode: Optional[str] = None,
        transport_mode: Optional[str] = None,
        avoid_features: Optional[List[str]] = None,
        avoid_areas: Optional[List[AvoidBoundingBox]] = None,
        truck: Optional[Truck] = None,
        matrix_attributes: Optional[List[str]] = None,
    ) -> Dict:
        path = "v8/matrix"
        url = f"{self._base_url}/{path}"
        params = {"async": async_req}
        data: Dict[Any, Any] = {
            "origins": origins,
        }
        if routing_mode:
            data["routingMode"] = routing_mode
        if transport_mode:
            data["transportMode"] = transport_mode

        regn_defn = vars(region_definition)
        data["regionDefinition"] = regn_defn
        if destinations:
            data["destinations"] = destinations
        if profile:
            data["profile"] = profile
        if departure_time:
            if isinstance(departure_time, datetime):
                data["departureTime"] = departure_time.isoformat(timespec="seconds")
            else:
                data["departureTime"] = departure_time
        if avoid_features and avoid_areas:
            avoid: Dict[str, Any] = {"features": avoid_features, "areas": []}
            for area in avoid_areas:
                avoid["areas"].append(vars(area))
            data["avoid"] = avoid
        if truck:
            data["truck"] = {k: v for k, v in vars(truck).items() if v is not None}
        if matrix_attributes:
            data["matrixAttributes"] = matrix_attributes
        resp = self.post(url, data=data, params=params)
        if resp.status_code in (200, 202):
            return resp.json()
        else:
            raise ApiError(resp)

    def matrix_route(
        self,
        origins: List[Dict],
        region_definition: Union[
            CircleRegion,
            BoundingBoxRegion,
            PolygonRegion,
            AutoCircleRegion,
            WorldRegion,
        ],
        destinations: Optional[List[Dict]] = None,
        profile: Optional[str] = None,
        departure_time: Optional[Union[datetime, str]] = None,
        routing_mode: Optional[str] = None,
        transport_mode: Optional[str] = None,
        avoid_features: Optional[List[str]] = None,
        avoid_areas: Optional[List[AvoidBoundingBox]] = None,
        truck: Optional[Truck] = None,
        matrix_attributes: Optional[List[str]] = None,
    ) -> Dict:
        """To Calculate routing matrix between multiple ``origins`` and ``destinations``
        synchronously.

        :param origins: A list of dictionaries containing lat and long for origin points.
        :param region_definition: Definition of a region in which the matrix will be calculated.
            Use object of atleast one of the following regions:
            :class:`here_location_services.config.matrix_routing_config.CircleRegion`
            :class:`here_location_services.config.matrix_routing_config.BoundingBoxRegion`
            :class:`here_location_services.config.matrix_routing_config.PolygonRegion`
            :class:`here_location_services.config.matrix_routing_config.AutoCircleRegion`
            :class:`here_location_services.config.matrix_routing_config.WorldRegion`
        :param destinations: A list of dictionaries containing lat and long for destination points.
            When no destinations are specified the matrix is assumed to be quadratic with origins
            used as destinations.
        :param profile: A string to represent profile id. A set predefined profile ids for route
            calculation can be used from config
            :attr:`PROFILE <here_location_services.config.matrix_routing_config.PROFILE>`
        :param departure_time: :class:`datetime.datetime` object.
        :param routing_mode: A string to represent routing mode. Routing mode values are defined
            in :attr:`ROUTING_MODE <here_location_services.config.routing_config.ROUTING_MODE>`
        :param transport_mode: A string to represent transport mode. Transport modes are defined
            in :attr:`ROUTING_TRANSPORT_MODE <here_location_services.config.routing_config.ROUTING_TRANSPORT_MODE>`
        :param avoid_features: Avoid routes that violate these properties. Avoid features are
            defined in :attr:`AVOID_FEATURES <here_location_services.config.matrix_routing_config.AVOID_FEATURES>`
        :param avoid_areas: A list of areas to avoid during route calculation. To define avoid area
            use object of :class:`AvoidBoundingBox here_location_services.config.matrix_routing_config.AvoidBoundingBox>`
        :param truck: Different truck options to use during route calculation when
            transportMode = truck. use object of :class:`Truck here_location_services.config.base_config.Truck>`
        :param matrix_attributes: Defines which attributes are included in the response as part of
            the data representation of the matrix entries summaries. Matrix attributes are defined
            in :attr:`MATRIX_ATTRIBUTES <here_location_services.config.matrix_routing_config.MATRIX_ATTRIBUTES>`
        :return: :class:`requests.Response` object.
        """  # noqa E501
        return self.__send_post_request(
            async_req="false",
            origins=origins,
            region_definition=region_definition,
            destinations=destinations,
            profile=profile,
            departure_time=departure_time,
            routing_mode=routing_mode,
            transport_mode=transport_mode,
            avoid_features=avoid_features,
            avoid_areas=avoid_areas,
            truck=truck,
            matrix_attributes=matrix_attributes,
        )

    def matrix_route_async(
        self,
        origins: List[Dict],
        region_definition: Union[
            CircleRegion,
            BoundingBoxRegion,
            PolygonRegion,
            AutoCircleRegion,
            WorldRegion,
        ],
        destinations: Optional[List[Dict]] = None,
        profile: Optional[str] = None,
        departure_time: Optional[Union[datetime, str]] = None,
        routing_mode: Optional[str] = None,
        transport_mode: Optional[str] = None,
        avoid_features: Optional[List[str]] = None,
        avoid_areas: Optional[List[AvoidBoundingBox]] = None,
        truck: Optional[Truck] = None,
        matrix_attributes: Optional[List[str]] = None,
    ) -> Dict:
        """To Calculate routing matrix between multiple ``origins`` and ``destinations``
        asynchronously.

        :param origins: A list of dictionaries containing lat and long for origin points.
        :param region_definition: Definition of a region in which the matrix will be calculated.
            Use object of atleast one of the following regions:
            :class:`here_location_services.config.matrix_routing_config.CircleRegion`
            :class:`here_location_services.config.matrix_routing_config.BoundingBoxRegion`
            :class:`here_location_services.config.matrix_routing_config.PolygonRegion`
            :class:`here_location_services.config.matrix_routing_config.AutoCircleRegion`
            :class:`here_location_services.config.matrix_routing_config.WorldRegion`
        :param destinations: A list of dictionaries containing lat and long for destination points.
            When no destinations are specified the matrix is assumed to be quadratic with origins
            used as destinations.
        :param profile: A string to represent profile id. A set predefined profile ids for route
            calculation can be used from config
            :attr:`PROFILE <here_location_services.config.matrix_routing_config.PROFILE>`
        :param departure_time: :class:`datetime.datetime` object.
        :param routing_mode: A string to represent routing mode. Routing mode values are defined
            in :attr:`ROUTING_MODE <here_location_services.config.routing_config.ROUTING_MODE>`
        :param transport_mode: A string to represent transport mode. Transport modes are defined
            in :attr:`ROUTING_TRANSPORT_MODE <here_location_services.config.routing_config.ROUTING_TRANSPORT_MODE>`
        :param avoid_features: Avoid routes that violate these properties. Avoid features are
            defined in :attr:`AVOID_FEATURES <here_location_services.config.matrix_routing_config.AVOID_FEATURES>`
        :param avoid_areas: A list of areas to avoid during route calculation. To define avoid area
            use object of :class:`AvoidBoundingBox here_location_services.config.matrix_routing_config.AvoidBoundingBox>`
        :param truck: Different truck options to use during route calculation when
            transportMode = truck. use object of :class:`Truck here_location_services.config.matrix_routing_config.Truck>`
        :param matrix_attributes: Defines which attributes are included in the response as part of
            the data representation of the matrix entries summaries. Matrix attributes are defined
            in :attr:`MATRIX_ATTRIBUTES <here_location_services.config.matrix_routing_config.MATRIX_ATTRIBUTES>`
        :return: :class:`requests.Response` object.
        """  # noqa E501
        return self.__send_post_request(
            async_req="true",
            origins=origins,
            region_definition=region_definition,
            destinations=destinations,
            profile=profile,
            departure_time=departure_time,
            routing_mode=routing_mode,
            transport_mode=transport_mode,
            avoid_features=avoid_features,
            avoid_areas=avoid_areas,
            truck=truck,
            matrix_attributes=matrix_attributes,
        )

    def get_async_matrix_route_status(self, status_url: str) -> requests.Response:
        """Get the status of async matrix calculation for the provided status url."""
        return self.get(status_url, allow_redirects=False)

    def get_async_matrix_route_results(self, result_url: str) -> requests.Response:
        """Get the results of async matrix calculation for the provided result url."""
        resp = self.get(result_url)
        if resp.status_code != 200:
            raise ApiError(resp)
        return resp.json()



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0

"""This module defines all the configs which will be required as inputs to autosuggest API."""

from .base_config import Bunch


class SearchCircle:
    """A class to define ``SearchCircle``

    Results will be returned if they are located within the specified circular area
        defined by its center and radius(in meters).
    """

    def __init__(self, lat: float, lng: float, radius: int):
        self.lat = lat
        self.lng = lng
        self.radius = radius


class PoliticalView(Bunch):
    """A Class to define constant values for political view

    ``RUS``:
    expressing the Russian view on Crimea

    ``SRB``:
    expressing the Serbian view on Kosovo, Vukovar and Sarengrad Islands

    ``MAR``:
    expressing the Moroccan view on Western Sahara
    """


#: Use this config for political_view of Autosuggest API.
#: Example: for ``RUS`` political_view use ``POLITICAL_VIEW.RUS``.
POLITICAL_VIEW = PoliticalView(
    **{
        "RUS": "RUS",
        "SRB": "SRB",
        "MAR": "MAR",
    }
)


class Show(Bunch):
    """A Class to define constant values for showing additional fields to be
    rendered in the response.

    ``phonemes``:
    Renders phonemes for address and place names into the results.

    ``tz``:
    BETA: Renders result items with additional time zone information.
    Please note that this may impact latency significantly.
    """


#: Use this config for show of Autosuggest API.
#: Example: for ``RUS`` show use ``SHOW.phonemes``.
SHOW = Show(**{"phonemes": "phonemes", "tz": "tz"})



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0

"""Project setup file."""

from codecs import open
from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# get the core dependencies and installs
with open(path.join(here, "requirements.txt"), encoding="utf-8") as f:
    all_reqs = f.read().split("\n")

install_requires = [x.strip() for x in all_reqs if "git+" not in x]
dependency_links = [
    x.strip().replace("git+", "") for x in all_reqs if x.startswith("git+")
]

# get extra dependencies
with open(path.join(here, "requirements_dev.txt"), encoding="utf-8") as f:
    dev_reqs = f.read().strip().split("\n")

packages = find_packages(exclude=["docs", "tests"])
version = {}
with open("{}/__version__.py".format(packages[0])) as f:
    exec(f.read(), version)

download_url = (
    "https://github.com/heremaps/here-location-services-python"
    "/archive/" + version["__version__"] + ".zip"
)


setup(
    version=version["__version__"],
    url="https://here.com",
    packages=packages,
    include_package_data=True,
    install_requires=install_requires,
    dependency_links=dependency_links,
    extras_require={"dev": dev_reqs},
    long_description=long_description,
    long_description_content_type="text/markdown",
)



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0

"""This module contains configurations for various Location services end points."""

row_conf = dict(here_api="hereapi.com")

china_conf = dict(here_api="hereapi.cn")

conf = dict(row=row_conf, china=china_conf)



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0

"""
This is a collection of utilities for using Here Location Services.
"""

import os
import warnings


def get_apikey() -> str:
    """
    Read and return the value of the environment variable ``LS_API_KEY``.

    :return: The string value of the environment variable or an empty string
        if no such variable could be found.
    """
    api_key = os.environ.get("LS_API_KEY")
    if api_key is None:
        warnings.warn("No token found in environment variable LS_API_KEY.")

    return api_key or ""



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0
"""This module will test platform api module."""
import pytest
from requests_oauthlib import OAuth1

from here_location_services.platform.apis.aaa_oauth2_api import AAAOauth2Api
from here_location_services.platform.apis.api import Api as PlaformApi
from here_location_services.utils import get_apikey
from tests.conftest import get_mock_response

LS_API_KEY = get_apikey()


def test_api_headers_property():
    api = PlaformApi(access_token="dummy")
    assert api.headers == {"Authorization": "Bearer dummy"}


def test_mock_request_post(mocker):
    mocker.patch("requests.post", return_value=True)
    api = PlaformApi(access_token="dummy")
    resp = api.post("dummy_url", data={"foo": "bar"})
    assert resp is True


def test_mock_request_scoped_access_token_excception(mocker):
    reason = "This is mock reason"
    text = "This is mock text"
    mock_response = get_mock_response(500, reason, text)
    mocker.patch("here_location_services.platform.apis.api.Api.post", return_value=mock_response)
    aaa_api = AAAOauth2Api(base_url="dummy")
    oauth = OAuth1(
        "dummy_key",
        client_secret="dummy_secret",
        signature_method="HMAC-SHA256",
    )
    with pytest.raises(Exception):
        aaa_api.request_scoped_access_token(oauth=oauth, data="dummy_data")



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0

"""This module contains classes for accessing `HERE Destination Weather API <https://developer.here.com/documentation/destination-weather/dev_guide/topics/overview.html>`_.
"""  # noqa E501

import time
from datetime import date, datetime
from typing import Any, Dict, List, Optional, Union

from geojson import Feature, FeatureCollection, LineString, MultiPolygon, Point, Polygon

from here_location_services.platform.auth import Auth

from .apis import Api
from .exceptions import ApiError


class DestinationWeatherApi(Api):
    """A class for accessing HERE routing APIs."""

    def __init__(
        self,
        api_key: Optional[str] = None,
        auth: Optional[Auth] = None,
        proxies: Optional[dict] = None,
        country: str = "row",
    ):
        super().__init__(api_key, auth=auth, proxies=proxies, country=country)
        self._base_url = f"https://weather.{self._get_url_string()}"

    def get_dest_weather(
        self,
        products: List[str],
        at: Optional[List] = None,
        query: Optional[str] = None,
        zipcode: Optional[str] = None,
        hourly_date: Optional[Union[date, datetime]] = None,
        one_observation: Optional[bool] = None,
        language: Optional[str] = None,
        units: Optional[str] = None,
    ):
        """Retrieves weather reports, weather forecasts, severe weather alerts and moon and sun rise and set information.

        See further information `Here Destination Weather API <https://developer.here.com/documentation/destination-weather/dev_guide/topics/overview.html>_`.

        :param products: List of :class:`DestWeatherProduct` identifying the type of
            report to obtain.
        :param at: A list of ``latitude`` and ``longitude`` specifying the area covered
            by the weather report.
        :param query: Free text query. Examples: "125, Berliner, berlin", "Beacon, Boston"
        :param zipcode: ZIP code of the location. This parameter is supported only for locations in
            the United States of America.
        :param hourly_date: Date for which hourly forecasts are to be retrieved. Can be either a `date` or
            `datetime` object
        :param one_observation: Boolean, if set to true, the response only includes the closest
            location. Only available when the `product` parameter is set to
            `DEST_WEATHER_PRODUCT.observation`.
        :param language: Defines the language used in the descriptions in the response.
        :param units: Defines whether units or imperial units are used in the response.
        :return: :class:`requests.Response` object.
        :raises ApiError: If ``status_code`` of API response is not 200.
        """  # noqa E501

        path = "v3/report"
        url = f"{self._base_url}/{path}"
        params: Dict[str, str] = {
            "products": ",".join([str(i) for i in products]),
        }
        if at:
            params["location"] = ",".join([str(i) for i in at])
        if query:
            params["q"] = query
        if zipcode:
            params["zipCode"] = zipcode
        if hourly_date:
            if type(hourly_date) is datetime.date:
                params["hourlyDate"] = hourly_date.strftime("%Y-%m-%d")
            else:
                params["hourlyDate"] = hourly_date.strftime("%Y-%m-%dT%H:%M:%S")
        if one_observation:
            params["oneObservation"] = "true" if one_observation else "false"
        if language:
            params["lang"] = language
        if units:
            params["units"] = units

        resp = self.get(url, params=params, proxies=self.proxies)
        if resp.status_code == 200:
            return resp
        else:
            raise ApiError(resp)

    def get_weather_alerts(
        self,
        geometry: Union[Point, LineString, Polygon, MultiPolygon],
        start_time: datetime,
        id: Optional[str] = None,
        weather_severity: Optional[int] = None,
        weather_type: Optional[str] = None,
        country: Optional[str] = None,
        end_time: Optional[datetime] = None,
        width: Optional[int] = 50000,
    ):
        """Retrieves weather reports, weather forecasts, severe weather alerts and moon and sun rise and set information.

        See further information `Here Destination Weather API <https://developer.here.com/documentation/destination-weather/dev_guide/topics/overview.html>_`.

        :param geometry: Point or LineString or Polygon or MultiPolygon defining the route or
            a single location
        :param start_time: Start time of the event
        :param id: Unique weather alert id.
        :param weather_severity: Defines the severity of the weather event as defined
            in :class:`WeatherSeverity`.
        :param weather_type: Defines the type of the weather event as defined
            in :class:`WeatherType`.
        :param country: String for ISO-3166-1 2-letter country code.
        :param end_time: End time of the event. If not present, warning is valid until
            it is not removed from the feed by national weather institutes
            (valid until warning is present in the response)
        :param width: int. default 50000
        :return: :class:`requests.Response` object.
        :raises ApiError: If ``status_code`` of API response is not 200.
        """  # noqa E501

        path = "v3/alerts"
        url = f"{self._base_url}/{path}"

        properties: Dict[str, Any] = {
            "width": width,
        }
        weather_warnings: Dict[str, Any] = {
            "startTime": time.mktime(start_time.timetuple()),
        }

        if weather_severity:
            weather_warnings["severity"] = weather_severity
        if weather_type:
            weather_warnings["type"] = weather_type
        if country:
            weather_warnings["country"] = country
        if end_time:
            weather_warnings["endTime"] = time.mktime(end_time.timetuple())

        properties["warnings"] = [weather_warnings]

        f = Feature(
            geometry=geometry,
            properties=properties,
        )

        if id:
            f.id = id

        feature_collection = FeatureCollection([])
        feature_collection.features.append(f)

        resp = self.post(url, data=feature_collection)
        if resp.status_code == 200:
            return resp
        else:
            raise ApiError(resp)



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0

"""
This module defines all the base classes which will be used for configuration classes of
various APIs.
"""

import json
from typing import List, Optional


class Bunch(dict):
    """A class for dot notation implementation of dictionary."""

    def __init__(self, **kwargs):
        dict.__init__(self, kwargs)
        self.__dict__ = self


class RoutingMode(Bunch):
    """A Class to define constant values for Routing Modes.

    ``fast``:
    Route calculation from start to destination optimized by travel time. In many cases, the route
    returned by the fast mode may not be the route with the fastest possible travel time.
    For example, the routing service may favor a route that remains on a highway, even
    if a faster travel time can be achieved by taking a detour or shortcut through
    an inconvenient side road.

    ``short``:
    Route calculation from start to destination disregarding any speed information. In this mode,
    the distance of the route is minimized, while keeping the route sensible. This includes,
    for example, penalizing turns. Because of that, the resulting route will not necessarily be
    the one with minimal distance.
    """


#: Use this config for routing_mode of routing API.
#: Example: for ``fast`` routing_mode use ``ROUTING_MODE.fast``.
ROUTING_MODE = RoutingMode(**{"fast": "fast", "short": "short"})


class ShippedHazardousGoods(Bunch):
    """A class to define the constant values for truck option ``shippedHazardousGoods``."""


#: Use this config for shipped_hazardous_goods attribute of Truck options of matrix Routing API.
#: Example: for ``explosive`` shipped_hazardous_goods  use ``SHIPPED_HAZARDOUS_GOODS.explosive``.
SHIPPED_HAZARDOUS_GOODS = ShippedHazardousGoods(
    **{
        "explosive": "explosive",
        "gas": "gas",
        "flammable": "flammable",
        "combustible": "combustible",
        "organic": "organic",
        "poison": "poison",
        "radioactive": "radioactive",
        "corrosive": "corrosive",
        "poisonousInhalation": "poisonousInhalation",
        "harmfulToWater": "harmfulToWater",
        "other": "other",
    }
)


class Truck:
    """A class to define different truck options which will be used during route calculation.
    Truck options should be used when transport_mode is ``truck``.
    """

    def __init__(
        self,
        shipped_hazardous_goods: Optional[List] = None,
        gross_weight: Optional[int] = None,
        weight_per_axle: Optional[int] = None,
        height: Optional[int] = None,
        width: Optional[int] = None,
        length: Optional[int] = None,
        tunnel_category: Optional[str] = None,
        axle_count: Optional[int] = None,
        truck_type: str = "straight",
        trailer_count: int = 0,
    ):
        """Object Initializer.

        :param shipped_hazardous_goods: List of hazardous materials in the vehicle. valid values
            for hazardous materials can be used from config
            :attr:`SHIPPED_HAZARDOUS_GOODS <here_location_services.config.matrix_routing_config.SHIPPED_HAZARDOUS_GOODS>`
        :param gross_weight: Total vehicle weight, including trailers and shipped goods, in
            kilograms. Should be greater than or equal to zero.
        :param weight_per_axle: Vehicle weight per axle, in kilograms. Should be greater than or
            equal to zero.
        :param height: Vehicle height, in centimeters. Should be in range [0, 5000]
        :param width: Vehicle width, in centimeters. Should be in range [0, 5000]
        :param length: Vehicle length, in centimeters. Should be in range [0, 5000]
        :param tunnel_category: A string for category of tunnel. Valid values are "B", "C", "D", "E".
            Specifies the `cargo tunnel restriction code <https://adrbook.com/en/2017/ADR/8.6.3>`_.
            The route will pass only through tunnels of less restrictive categories.
        :param axle_count: Total number of axles that the vehicle has. Should be in the
            range [2, 255].
        :param truck_type: A string to represent the type of truck.
        :param trailer_count: Number of trailers attached to the vehicle.
        """  # noqa E501
        self.shippedHazardousGoods = shipped_hazardous_goods
        self.grossWeight = gross_weight
        self.weightPerAxle = weight_per_axle
        self.height = height
        self.width = width
        self.length = length
        self.tunnelCategory = tunnel_category
        self.axleCount = axle_count
        self.type = truck_type
        self.trailerCount = trailer_count


class PlaceOptions:
    """A class to define ``PlaceOptions`` for ``origin``/ ``via``/ ``destination``.

    Various options can be found here:

    `PlaceOptions <https://developer.here.com/documentation/routing-api/8.16.0/api-reference-swagger.html>`_.
    """  # noqa E501

    def __init__(
        self,
        course: Optional[int] = None,
        sideof_street_hint: Optional[List[float]] = None,
        match_sideof_street: Optional[str] = None,
        namehint: Optional[str] = None,
        radius: Optional[int] = None,
        min_course_distance: Optional[int] = None,
    ):
        """Object Initializer.

        :param course: An int representing degrees clock-wise from north.
            Indicating the desired direction at the place. E.g. 90 indicating ``east``.
            This is defined in constant ``ROUTE_COURSE``.
        :param sideof_street_hint: A list of latitude and longitude.Indicating the side of the
            street that should be used.
        :param match_sideof_street: Specifies how the location set by ``sideof_street_hint`` should
            be handled. If this is set then sideof_street_hint should also be set. There are two
            valid values for match_sideof_street:

            ``always``:
            Always prefer the given side of street.

            ``onlyIfDivided``:
            Only prefer using side of street set by ``sideof_street_hint`` in case the street
            has dividers. This is the default behavior.

            These values are mainted as config in:
            :attr:`ROUTE_MATCH_SIDEOF_STREET <here_location_services.config.routing_config.ROUTE_MATCH_SIDEOF_STREET>`
        :param namehint: A string for the router to look for the place with the most similar name.
            This can e.g. include things like: North being used to differentiate between
            interstates I66 North and I66 South, Downtown Avenue being used to correctly
            select a residential street.
        :param radius: In meters Asks the router to consider all places within the given radius as
            potential candidates for route calculation. This can be either because it is not
            important which place is used, or because it is unknown. Radius more than 200 meters
            are not supported.
        :param min_course_distance: In meters Asks the routing service to try to find a route that
            avoids actions for the indicated distance. E.g. if the origin is determined by a moving
            vehicle, the user might not have time to react to early actions.
        """  # noqa E501
        self.course = course
        self.sideOfStreetHint: Optional[str] = None
        if sideof_street_hint is not None:
            self.sideOfStreetHint = ",".join([str(point) for point in sideof_street_hint])
        self.matchSideOfStreet = match_sideof_street
        self.namehint = namehint
        self.radius = radius
        self.minCourseDistance = min_course_distance

    def __repr__(self):
        """Return string representation of this instance."""
        return json.dumps(self.__dict__)


class WayPointOptions:
    """A class to define ``PlaceOptions`` for ``via``/ ``destination``.

    Various options can be found here:

    `PlaceOptions <https://developer.here.com/documentation/routing-api/8.16.0/api-reference-swagger.html>`_.
    """  # noqa E501

    def __init__(self, stop_duration: Optional[int] = None, pass_through: Optional[bool] = None):
        self.stopDuration = stop_duration
        self.passThrough = pass_through

    def __repr__(self):
        """Return string representation of this instance."""
        return json.dumps(self.__dict__)



# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0
"""This module will test Location services APIs using auth token for authentication."""
import os
from datetime import datetime

import pytest
import pytz

from here_location_services import LS
from here_location_services.config.base_config import ROUTING_MODE
from here_location_services.config.matrix_routing_config import (
    AVOID_FEATURES,
    MATRIX_ATTRIBUTES,
    AvoidBoundingBox,
    WorldRegion,
)
from here_location_services.config.routing_config import ROUTING_TRANSPORT_MODE
from here_location_services.platform.credentials import PlatformCredentials
from here_location_services.responses import GeocoderResponse
from tests.conftest import env_setup_done

auth_present = env_setup_done()


@pytest.mark.skipif(not auth_present, reason="Credentials are not setup in env.")
def test_ls_geocoding_auth_token():
    """Test geocoding api."""
    api_key = os.environ.get("LS_API_KEY")
    try:
        del os.environ["LS_API_KEY"]
        address = "200 S Mathilda Sunnyvale CA"
        credentials = PlatformCredentials.from_default()
        ls = LS(platform_credentials=credentials)
        resp = ls.geocode(query=address, limit=2)
        assert isinstance(resp, GeocoderResponse)
        assert resp.__str__()
        assert resp.as_json_string()
        geo_json = resp.to_geojson()
        assert geo_json.type == "FeatureCollection"
        assert geo_json.features
        pos = resp.items[0]["position"]
        assert len(resp.items) == 1
        assert pos == {"lat": 37.37634, "lng": -122.03405}
    finally:
        os.environ["LS_API_KEY"] = api_key


@pytest.mark.skipif(not auth_present, reason="Credentials are not setup in env.")
def test_matrix_route_auth_token():
    """Test Matrix routing using auth token."""
    platform_credentials = PlatformCredentials.from_default()
    ls = LS(platform_credentials=platform_credentials)
    origins = [
        {"lat": 37.76, "lng": -122.42},
        {"lat": 40.63, "lng": -74.09},
        {"lat": 30.26, "lng": -97.74},
    ]
    region_definition = WorldRegion()
    matrix_attributes = [MATRIX_ATTRIBUTES.distances, MATRIX_ATTRIBUTES.travelTimes]
    avoid_areas = AvoidBoundingBox(68.1766451354, 7.96553477623, 97.4025614766, 35.4940095078)
    result = ls.matrix(
        origins=origins,
        region_definition=region_definition,
        destinations=origins,
        routing_mode=ROUTING_MODE.fast,
        departure_time=datetime.now(tz=pytz.utc),
        transport_mode=ROUTING_TRANSPORT_MODE.truck,
        avoid_features=[AVOID_FEATURES.tollRoad],
        avoid_areas=[avoid_areas],
        matrix_attributes=matrix_attributes,
    )
    mat = result.matrix
    assert mat["numOrigins"] == 3
    assert mat["numDestinations"] == 3
    assert len(mat["distances"]) == 9
